@@ -1,6 +1,6 @@
 /*
  * mpstat: per-processor statistics
- * (C) 2000-2003 by Sebastien GODARD <sebastien.godard@wanadoo.fr>
+ * (C) 2000-2004 by Sebastien GODARD <sebastien.godard@wanadoo.fr>
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -49,30 +49,32 @@ struct mp_stats *st_mp_cpu[DIM];
 unsigned char *cpu_bitmap;	/* Bit 0: Global; Bit 1: 1st proc; etc. */
 struct mp_timestamp st_mp_tstamp[DIM];
 /* Nb of processors on the machine. A value of 1 means two processors */
-int proc_used = -1;
+int cpu_nr = -1;
 long interval = 0, count = 0;
 unsigned int flags = 0;
-struct tm loc_time;
 
 
 /*
+ ***************************************************************************
  * Print usage and exit
+ ***************************************************************************
  */
 void usage(char *progname)
 {
    fprintf(stderr, _("sysstat version %s\n"
-		   "(C) S. Godard <sebastien.godard@wanadoo.fr>\n"
-	           "Usage: %s [ options... ]\n"
+		   "(C) Sebastien Godard\n"
+	           "Usage: %s [ options... ] [ <interval> [ <count> ] ]\n"
 		   "Options are:\n"
-		   "[ -P { <cpu> | ALL } ] [ -V ]\n"
-		   "[ <interval> [ <count> ] ]\n"),
+		   "[ -P { <cpu> | ALL } ] [ -V ]\n"),
 	   VERSION, progname);
    exit(1);
 }
 
 
 /*
+ ***************************************************************************
  * SIGALRM signal handler
+ ***************************************************************************
  */
 void alarm_handler(int sig)
 {
@@ -82,7 +84,9 @@ void alarm_handler(int sig)
 
 
 /*
+ ***************************************************************************
  * Allocate mp_stats structures and cpu bitmap
+ ***************************************************************************
  */
 void salloc_mp_cpu(int nr_cpus)
 {
@@ -107,71 +111,93 @@ void salloc_mp_cpu(int nr_cpus)
 
 
 /*
- * Print statistics average
+ ***************************************************************************
+ * Core function used to display statistics
+ ***************************************************************************
  */
-void write_stats_avg(short curr, short dis)
+void write_stats_core(short prev, short curr, short dis,
+		      char *prev_string, char *curr_string)
 {
    struct mp_stats *st_mp_cpu_i, *st_mp_cpu_j;
    unsigned long itv;
    int cpu;
 
+   /*
+    * Under very special circumstances, STDOUT may become unavailable,
+    * This is what we try to guess here
+    */
+   if (write(STDOUT_FILENO, "", 0) == -1) {
+      perror("stdout");
+      exit(6);
+   }
+
    /* Interval value in jiffies, multiplied by the number of proc */
-   itv = st_mp_tstamp[curr].uptime - st_mp_tstamp[2].uptime;
+   itv = st_mp_tstamp[curr].uptime - st_mp_tstamp[prev].uptime;
 
    if (!itv)	/* Paranoia checking */
       itv = 1;
 
    /* Print stats */
    if (dis)
-      printf(_("\nAverage:     CPU   %%user   %%nice %%system %%iowait   %%idle    intr/s\n"));
+      printf(_("\n%-11s  CPU   %%user   %%nice %%system %%iowait    %%irq   %%soft   %%idle    intr/s\n"),
+	     prev_string);
 
-   /* Check if we want global average stats among all proc */
+   /* Check if we want global stats among all proc */
    if (*cpu_bitmap & 1) {
 
-      printf(_("Average:     all"));
+      printf(_("%-11s  all"), curr_string);
 
-      printf("  %6.2f  %6.2f  %6.2f  %6.2f",
-	     SP_VALUE(st_mp_cpu[2]->cpu_user,   st_mp_cpu[curr]->cpu_user,   itv),
-	     SP_VALUE(st_mp_cpu[2]->cpu_nice,   st_mp_cpu[curr]->cpu_nice,   itv),
-	     SP_VALUE(st_mp_cpu[2]->cpu_system, st_mp_cpu[curr]->cpu_system, itv),
-	     SP_VALUE(st_mp_cpu[2]->cpu_iowait, st_mp_cpu[curr]->cpu_iowait, itv));
+      printf("  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f",
+	     SP_VALUE(st_mp_cpu[prev]->cpu_user,    st_mp_cpu[curr]->cpu_user,    itv),
+	     SP_VALUE(st_mp_cpu[prev]->cpu_nice,    st_mp_cpu[curr]->cpu_nice,    itv),
+	     SP_VALUE(st_mp_cpu[prev]->cpu_system,  st_mp_cpu[curr]->cpu_system,  itv),
+	     SP_VALUE(st_mp_cpu[prev]->cpu_iowait,  st_mp_cpu[curr]->cpu_iowait,  itv),
+	     SP_VALUE(st_mp_cpu[prev]->cpu_hardirq, st_mp_cpu[curr]->cpu_hardirq, itv),
+	     SP_VALUE(st_mp_cpu[prev]->cpu_softirq, st_mp_cpu[curr]->cpu_softirq, itv));
 
-      if (st_mp_cpu[curr]->cpu_idle < st_mp_cpu[2]->cpu_idle)
-	 printf("    %.2f", 0.0);	/* Handle buggy RTC (or kernels?) */
+      if (st_mp_cpu[curr]->cpu_idle < st_mp_cpu[prev]->cpu_idle)
+	 printf("    %.2f", 0.0);	/* Handle buggy kernels */
       else
 	 printf("  %6.2f",
-		SP_VALUE(st_mp_cpu[2]->cpu_idle, st_mp_cpu[curr]->cpu_idle, itv));
+		SP_VALUE(st_mp_cpu[prev]->cpu_idle, st_mp_cpu[curr]->cpu_idle, itv));
    }
 
-   /* Here, we reduce the interval value to one processor! */
-   itv /= (proc_used + 1);
-   if (!itv)
-      itv = 1;
+   /*
+    * Here, we reduce the interval value to one processor,
+    * using the uptime computed for proc#0.
+    */
+   if (cpu_nr) {
+      itv = st_mp_tstamp[curr].uptime0 - st_mp_tstamp[prev].uptime0;
+      if (!itv)
+	 itv = 1;
+   }
 
    if (*cpu_bitmap & 1) {
       printf(" %9.2f\n",
-	  S_VALUE(st_mp_cpu[2]->irq, st_mp_cpu[curr]->irq, itv));
+	  S_VALUE(st_mp_cpu[prev]->irq, st_mp_cpu[curr]->irq, itv));
    }
 
-   for (cpu = 1; cpu <= proc_used + 1; cpu++) {
+   for (cpu = 1; cpu <= cpu_nr + 1; cpu++) {
 
       /* Check if we want stats about this proc */
       if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))))
 	 continue;
 
-      printf(_("Average:    %4d"), cpu - 1);
+      printf("%-11s %4d", curr_string, cpu - 1);
 
-      st_mp_cpu_i = st_mp_cpu[curr]  + cpu;
-      st_mp_cpu_j = st_mp_cpu[2] + cpu;
+      st_mp_cpu_i = st_mp_cpu[curr] + cpu;
+      st_mp_cpu_j = st_mp_cpu[prev] + cpu;
 
-      printf("  %6.2f  %6.2f  %6.2f  %6.2f",
-	     SP_VALUE(st_mp_cpu_j->cpu_user,   st_mp_cpu_i->cpu_user,   itv),
-	     SP_VALUE(st_mp_cpu_j->cpu_nice,   st_mp_cpu_i->cpu_nice,   itv),
-	     SP_VALUE(st_mp_cpu_j->cpu_system, st_mp_cpu_i->cpu_system, itv),
-	     SP_VALUE(st_mp_cpu_j->cpu_iowait, st_mp_cpu_i->cpu_iowait, itv));
+      printf("  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f",
+	     SP_VALUE(st_mp_cpu_j->cpu_user,    st_mp_cpu_i->cpu_user,    itv),
+	     SP_VALUE(st_mp_cpu_j->cpu_nice,    st_mp_cpu_i->cpu_nice,    itv),
+	     SP_VALUE(st_mp_cpu_j->cpu_system,  st_mp_cpu_i->cpu_system,  itv),
+	     SP_VALUE(st_mp_cpu_j->cpu_iowait,  st_mp_cpu_i->cpu_iowait,  itv),
+	     SP_VALUE(st_mp_cpu_j->cpu_hardirq, st_mp_cpu_i->cpu_hardirq, itv),
+	     SP_VALUE(st_mp_cpu_j->cpu_softirq, st_mp_cpu_i->cpu_softirq, itv));
 
       if (st_mp_cpu_i->cpu_idle < st_mp_cpu_j->cpu_idle)
-	 printf("    %.2f", 0.0);	/* Handle buggy RTC (or kernels?) */
+	 printf("    %.2f", 0.0);
       else
 	 printf("  %6.2f",
 		SP_VALUE(st_mp_cpu_j->cpu_idle, st_mp_cpu_i->cpu_idle, itv));
@@ -183,111 +209,62 @@ void write_stats_avg(short curr, short dis)
 
 
 /*
- * Print statistics
+ ***************************************************************************
+ * Print statistics average
+ ***************************************************************************
  */
-void write_stats(short curr, short dis)
+void write_stats_avg(short curr, short dis)
+{
+   char string[16];
+
+   strcpy(string, _("Average:"));
+   write_stats_core(2, curr, dis, string, string);
+}
+
+
+/*
+ ***************************************************************************
+ * Print statistics
+ ***************************************************************************
+ */
+void write_stats(short curr, short dis, struct tm *loc_time)
 {
    char cur_time[2][14];
-   struct mp_stats *st_mp_cpu_i, *st_mp_cpu_j;
-   unsigned long itv;
-   int cpu;
 
    /*
     * Get previous timestamp
     * NOTE: loc_time structure must have been init'ed before!
     */
-   loc_time.tm_hour = st_mp_tstamp[!curr].hour;
-   loc_time.tm_min  = st_mp_tstamp[!curr].minute;
-   loc_time.tm_sec  = st_mp_tstamp[!curr].second;
-   strftime(cur_time[!curr], 14, "%X  ", &loc_time);
+   loc_time->tm_hour = st_mp_tstamp[!curr].hour;
+   loc_time->tm_min  = st_mp_tstamp[!curr].minute;
+   loc_time->tm_sec  = st_mp_tstamp[!curr].second;
+   strftime(cur_time[!curr], 14, "%X  ", loc_time);
 
    /* Get current timestamp */
-   loc_time.tm_hour = st_mp_tstamp[curr].hour;
-   loc_time.tm_min  = st_mp_tstamp[curr].minute;
-   loc_time.tm_sec  = st_mp_tstamp[curr].second;
-   strftime(cur_time[curr], 14, "%X  ", &loc_time);
+   loc_time->tm_hour = st_mp_tstamp[curr].hour;
+   loc_time->tm_min  = st_mp_tstamp[curr].minute;
+   loc_time->tm_sec  = st_mp_tstamp[curr].second;
+   strftime(cur_time[curr], 14, "%X  ", loc_time);
 
    /* Only the first 11 characters are printed */
    cur_time[curr][11] = cur_time[!curr][11] = '\0';
 
-   /* Interval value in jiffies, multiplied by the number of proc */
-   itv = st_mp_tstamp[curr].uptime - st_mp_tstamp[!curr].uptime;
-
-   if (!itv)	/* Paranoia checking */
-      itv = 1;
-
-   /* Print stats */
-   if (dis)
-      printf(_("\n%-11s  CPU   %%user   %%nice %%system %%iowait   %%idle    intr/s\n"),
-	     cur_time[!curr]);
-
-   /* Check if we want global stats among all proc */
-   if (*cpu_bitmap & 1) {
-
-      printf(_("%-11s  all"), cur_time[curr]);
-
-      printf("  %6.2f  %6.2f  %6.2f  %6.2f",
-	     SP_VALUE(st_mp_cpu[!curr]->cpu_user,   st_mp_cpu[curr]->cpu_user,   itv),
-	     SP_VALUE(st_mp_cpu[!curr]->cpu_nice,   st_mp_cpu[curr]->cpu_nice,   itv),
-	     SP_VALUE(st_mp_cpu[!curr]->cpu_system, st_mp_cpu[curr]->cpu_system, itv),
-	     SP_VALUE(st_mp_cpu[!curr]->cpu_iowait, st_mp_cpu[curr]->cpu_iowait, itv));
-
-      if (st_mp_cpu[curr]->cpu_idle < st_mp_cpu[!curr]->cpu_idle)
-	 printf("    %.2f", 0.0);	/* Handle buggy RTC (or kernels?) */
-      else
-	 printf("  %6.2f",
-		SP_VALUE(st_mp_cpu[!curr]->cpu_idle, st_mp_cpu[curr]->cpu_idle, itv));
-   }
-
-   /* Here, we reduce the interval value to one processor! */
-   itv /= (proc_used + 1);
-   if (!itv)
-      itv = 1;
-
-   if (*cpu_bitmap & 1) {
-      printf(" %9.2f\n",
-	  S_VALUE(st_mp_cpu[!curr]->irq, st_mp_cpu[curr]->irq, itv));
-   }
-
-   for (cpu = 1; cpu <= proc_used + 1; cpu++) {
-
-      /* Check if we want stats about this proc */
-      if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))))
-	 continue;
-
-      printf("%-11s %4d", cur_time[curr], cpu - 1);
-
-      st_mp_cpu_i = st_mp_cpu[curr]  + cpu;
-      st_mp_cpu_j = st_mp_cpu[!curr] + cpu;
-
-      printf("  %6.2f  %6.2f  %6.2f  %6.2f",
-	     SP_VALUE(st_mp_cpu_j->cpu_user,   st_mp_cpu_i->cpu_user,   itv),
-	     SP_VALUE(st_mp_cpu_j->cpu_nice,   st_mp_cpu_i->cpu_nice,   itv),
-	     SP_VALUE(st_mp_cpu_j->cpu_system, st_mp_cpu_i->cpu_system, itv),
-	     SP_VALUE(st_mp_cpu_j->cpu_iowait, st_mp_cpu_i->cpu_iowait, itv));
-
-      if (st_mp_cpu_i->cpu_idle < st_mp_cpu_j->cpu_idle)
-	 printf("    %.2f", 0.0);	/* Handle buggy RTC (or kernels?) */
-      else
-	 printf("  %6.2f",
-		SP_VALUE(st_mp_cpu_j->cpu_idle, st_mp_cpu_i->cpu_idle, itv));
-
-      printf(" %9.2f\n",
-	  S_VALUE(st_mp_cpu_j->irq, st_mp_cpu_i->irq, itv));
-   }
+   write_stats_core(!curr, curr, dis, cur_time[!curr], cur_time[curr]);
 }
 
 
 /*
+ ***************************************************************************
  * Read stats from /proc/stat
  * (see linux source file linux/fs/proc/array.c)
+ ***************************************************************************
  */
 void read_proc_stat(short curr)
 {
    FILE *statfp;
    struct mp_stats *st_mp_cpu_i;
    static char line[80];
-   unsigned int cc_user, cc_nice, cc_system;
+   unsigned int cc_user, cc_nice, cc_system, cc_hardirq, cc_softirq;
    unsigned long cc_idle, cc_iowait;
    int proc_nb;
 
@@ -301,15 +278,20 @@ void read_proc_stat(short curr)
 
       if (!strncmp(line, "cpu ", 4)) {
 	 /*
-	  * Read the number of jiffies spent in user, nice, system, idle
-	  * and iowait mode among all proc. CPU usage is not reduced to one
+	  * Read the number of jiffies spent in the different modes
+	  * among all proc. CPU usage is not reduced to one
 	  * processor to avoid rounding problems.
 	  */
-	 st_mp_cpu[curr]->cpu_iowait = 0;	/* For non 2.5 machines */
-	 sscanf(line + 5, "%u %u %u %lu %lu",
-		&(st_mp_cpu[curr]->cpu_user),   &(st_mp_cpu[curr]->cpu_nice),
-		&(st_mp_cpu[curr]->cpu_system), &(st_mp_cpu[curr]->cpu_idle),
-		&(st_mp_cpu[curr]->cpu_iowait));
+	 st_mp_cpu[curr]->cpu_iowait = 0;	/* For pre 2.5 kernels */
+	 cc_hardirq = cc_softirq = 0;
+	 sscanf(line + 5, "%u %u %u %lu %lu %u %u",
+		&(st_mp_cpu[curr]->cpu_user),
+		&(st_mp_cpu[curr]->cpu_nice),
+		&(st_mp_cpu[curr]->cpu_system),
+		&(st_mp_cpu[curr]->cpu_idle),
+		&(st_mp_cpu[curr]->cpu_iowait),
+		&(st_mp_cpu[curr]->cpu_hardirq),
+		&(st_mp_cpu[curr]->cpu_softirq));
 
 	 /*
 	  * Compute the uptime of the system in jiffies (1/100ths of a second
@@ -320,31 +302,46 @@ void read_proc_stat(short curr)
 	                             st_mp_cpu[curr]->cpu_nice +
 	                             st_mp_cpu[curr]->cpu_system +
 	                             st_mp_cpu[curr]->cpu_idle +
-	                             st_mp_cpu[curr]->cpu_iowait;
+	                             st_mp_cpu[curr]->cpu_iowait +
+	    			     st_mp_cpu[curr]->cpu_hardirq +
+	    			     st_mp_cpu[curr]->cpu_softirq;
       }
 
       else if (!strncmp(line, "cpu", 3)) {
 	 /*
-	  * Read the number of jiffies spent in user, nice, system, idle
-	  * and iowait mode for current proc.
+	  * Read the number of jiffies spent in the different modes
+	  * (user, nice, etc.) for current proc.
 	  * This is done only on SMP machines.
 	  */
-	 cc_iowait = 0;	/* For non 2.5 machines */
-	 sscanf(line + 3, "%d %u %u %u %lu %lu",
+	 cc_iowait = cc_hardirq = cc_softirq = 0;
+	 sscanf(line + 3, "%d %u %u %u %lu %lu %u %u",
 		&proc_nb,
-		&cc_user, &cc_nice, &cc_system, &cc_idle, &cc_iowait);
-	 if (proc_nb <= proc_used) {
+		&cc_user, &cc_nice, &cc_system, &cc_idle, &cc_iowait,
+		&cc_hardirq, &cc_softirq);
+
+	 if (proc_nb <= cpu_nr) {
 	    st_mp_cpu_i = st_mp_cpu[curr] + proc_nb + 1;
-	    st_mp_cpu_i->cpu_user   = cc_user;
-	    st_mp_cpu_i->cpu_nice   = cc_nice;
-	    st_mp_cpu_i->cpu_system = cc_system;
-	    st_mp_cpu_i->cpu_idle   = cc_idle;
-	    st_mp_cpu_i->cpu_iowait = cc_iowait;
+	    st_mp_cpu_i->cpu_user    = cc_user;
+	    st_mp_cpu_i->cpu_nice    = cc_nice;
+	    st_mp_cpu_i->cpu_system  = cc_system;
+	    st_mp_cpu_i->cpu_idle    = cc_idle;
+	    st_mp_cpu_i->cpu_iowait  = cc_iowait;
+	    st_mp_cpu_i->cpu_hardirq = cc_hardirq;
+	    st_mp_cpu_i->cpu_softirq = cc_softirq;
 	 }
 	 /* else:
 	  * Additional CPUs have been dynamically registered in /proc/stat.
 	  * mpstat won't crash, but the CPU stats might be false...
 	  */
+	
+	 if (!proc_nb)
+	    /*
+	     * Compute uptime reduced for one proc,
+	     * using jiffies count for proc#0.
+	     */
+	    st_mp_tstamp[curr].uptime0 = cc_user + cc_nice + cc_system +
+	       				 cc_idle + cc_iowait + cc_hardirq +
+	    				 cc_softirq;
       }
 
       else if (!strncmp(line, "intr ", 5))
@@ -358,16 +355,18 @@ void read_proc_stat(short curr)
 
 
 /*
+ ***************************************************************************
  * Read stats from /proc/interrupts
+ ***************************************************************************
  */
 void read_interrupts_stat(short curr)
 {
    FILE *irqfp;
    struct mp_stats *st_mp_cpu_i;
-   static char line[512];	/* Should depend on the nb of processors */
+   static char line[INTERRUPTS_LINE];
    unsigned int irq = 0, cpu;
 
-   for (cpu = 0; cpu <= proc_used; cpu++) {
+   for (cpu = 0; cpu <= cpu_nr; cpu++) {
       st_mp_cpu_i = st_mp_cpu[curr] + cpu +1;
       st_mp_cpu_i->irq = 0;
    }
@@ -375,11 +374,11 @@ void read_interrupts_stat(short curr)
    /* Open interrupts file */
    if ((irqfp = fopen(INTERRUPTS, "r")) != NULL) {
 
-      while (fgets(line, 512, irqfp) != NULL) {
+      while (fgets(line, INTERRUPTS_LINE, irqfp) != NULL) {
 
 	 if (isdigit(line[2])) {
 	
-	    for (cpu = 0; cpu <= proc_used; cpu++) {
+	    for (cpu = 0; cpu <= cpu_nr; cpu++) {
 	       st_mp_cpu_i = st_mp_cpu[curr] + cpu + 1;
 	       sscanf(line + 4 + 11 * cpu, " %10u", &irq);
 	       st_mp_cpu_i->irq += irq;
@@ -394,15 +393,70 @@ void read_interrupts_stat(short curr)
 
 
 /*
+ ***************************************************************************
+ * Main loop: read stats from the relevant sources,
+ * and display them.
+ ***************************************************************************
+ */
+void rw_mp_stat_loop(short dis_hdr, unsigned long lines, int rows,
+		     struct tm *loc_time)
+{
+   short curr = 1, dis = 1;
+
+   do {
+
+      /* Resetting the structure not needed since every fields will be set */
+
+      /* Save time */
+      get_localtime(loc_time);
+
+      st_mp_tstamp[curr].hour   = loc_time->tm_hour;
+      st_mp_tstamp[curr].minute = loc_time->tm_min;
+      st_mp_tstamp[curr].second = loc_time->tm_sec;
+
+      /* Read stats */
+      read_proc_stat(curr);
+      read_interrupts_stat(curr);
+
+      /* Write stats */
+      if (!dis_hdr) {
+	 dis = lines / rows;
+	 if (dis)
+	    lines %= rows;
+	 lines++;
+      }
+      write_stats(curr, dis, loc_time);
+
+      /* Flush data */
+      fflush(stdout);
+
+      if (count > 0)
+	 count--;
+      if (count) {
+	 curr ^= 1;
+	 pause();
+      }
+   }
+   while (count);
+
+   /* Write stats average */
+   write_stats_avg(curr, dis_hdr);
+}
+
+
+/*
+ ***************************************************************************
  * Main entry to the program
+ ***************************************************************************
  */
 int main(int argc, char **argv)
 {
    int opt = 0, i;
    struct utsname header;
-   short curr = 1, dis_hdr = -1, opt_used = 0, dis = 1;
+   short dis_hdr = -1, opt_used = 0;
    unsigned long lines = 0;
    int rows = 23;
+   struct tm loc_time;
 
 #ifdef USE_NLS
    /* Init National Language Support */
@@ -410,13 +464,13 @@ int main(int argc, char **argv)
 #endif
 
    /* How many processors on this machine ? */
-   get_nb_proc_used(&proc_used, ~0);
+   get_cpu_nr(&cpu_nr, ~0);
 
    /*
-    * proc_used: a value of 1 means there are 2 processors (0 and 1).
+    * cpu_nr: a value of 1 means there are 2 processors (0 and 1).
     * In this case, we have to allocate 3 structures: global, proc0 and proc1.
     */
-   salloc_mp_cpu(proc_used + 2);
+   salloc_mp_cpu(cpu_nr + 2);
 
    while (++opt < argc) {
 
@@ -429,19 +483,19 @@ int main(int argc, char **argv)
 	    opt_used = 1;
 	    dis_hdr++;
 	    if (!strcmp(argv[opt], K_ALL)) {
-	       if (proc_used)
+	       if (cpu_nr)
 		  dis_hdr = 9;
 	       /*
 		* Set bit for every processor.
 		* Also indicate to display stats for CPU 'all'.
 		*/
-	       memset(cpu_bitmap, 0xff, ((proc_used + 1 + (proc_used > 0)) >> 3) + 1);
+	       memset(cpu_bitmap, 0xff, ((cpu_nr + 1 + (cpu_nr > 0)) >> 3) + 1);
 	    }
 	    else {
 	       if (strspn(argv[opt], DIGITS) != strlen(argv[opt]))
 		  usage(argv[0]);
 	       i = atoi(argv[opt]);	/* Get cpu number */
-	       if (i > proc_used) {
+	       if (i > cpu_nr) {
 		  fprintf(stderr, _("Not that many processors!\n"));
 		  exit(1);
 	       }
@@ -505,15 +559,12 @@ int main(int argc, char **argv)
 
    if (!interval || WANT_BOOT_STATS(flags)) {
       /* Display since boot time */
-      st_mp_tstamp[1].hour   = st_mp_tstamp[0].hour;
-      st_mp_tstamp[1].minute = st_mp_tstamp[0].minute;
-      st_mp_tstamp[1].second = st_mp_tstamp[0].second;
+      st_mp_tstamp[1] = st_mp_tstamp[0];
+      st_mp_tstamp[1].uptime = st_mp_tstamp[1].uptime0 = 0;
 
-      st_mp_tstamp[1].uptime = 0;
+      memset(st_mp_cpu[1], 0, MP_STATS_SIZE * (cpu_nr + 2));
 
-      memset(st_mp_cpu[1], 0, MP_STATS_SIZE * (proc_used + 2));
-
-      write_stats(0, DISP_HDR);
+      write_stats(0, DISP_HDR, &loc_time);
       exit(0);
    }
 
@@ -521,55 +572,13 @@ int main(int argc, char **argv)
    alarm_handler(0);
 
    /* Save the first stats collected. Will be used to compute the average */
-   st_mp_tstamp[2].hour   = st_mp_tstamp[0].hour;
-   st_mp_tstamp[2].minute = st_mp_tstamp[0].minute;
-   st_mp_tstamp[2].second = st_mp_tstamp[0].second;
-
-   st_mp_tstamp[2].uptime = st_mp_tstamp[0].uptime;
-
-   memcpy(st_mp_cpu[2], st_mp_cpu[0], MP_STATS_SIZE * (proc_used + 2));
+   st_mp_tstamp[2] = st_mp_tstamp[0];
+   memcpy(st_mp_cpu[2], st_mp_cpu[0], MP_STATS_SIZE * (cpu_nr + 2));
 
    pause();
 
    /* Main loop */
-   do {
-
-      /* Resetting the structure not needed since every fields will be set */
-
-      /* Save time */
-      get_localtime(&loc_time);
-
-      st_mp_tstamp[curr].hour   = loc_time.tm_hour;
-      st_mp_tstamp[curr].minute = loc_time.tm_min;
-      st_mp_tstamp[curr].second = loc_time.tm_sec;
-
-      /* Read stats */
-      read_proc_stat(curr);
-      read_interrupts_stat(curr);
-
-      /* Write stats */
-      if (!dis_hdr) {
-	 dis = lines / rows;
-	 if (dis)
-	    lines %= rows;
-	 lines++;
-      }
-      write_stats(curr, dis);
-
-      /* Flush data */
-      fflush(stdout);
-
-      if (count > 0)
-	 count--;
-      if (count) {
-	 curr ^= 1;
-	 pause();
-      }
-   }
-   while (count);
-
-   /* Write stats average */
-   write_stats_avg(curr, dis_hdr);
+   rw_mp_stat_loop(dis_hdr, lines, rows, &loc_time);
 
    return 0;
 }
