@@ -1,6 +1,6 @@
 /*
  * sar/sadc: report system activity
- * (C) 1999-2003 by Sebastien Godard <sebastien.godard@wanadoo.fr>
+ * (C) 1999-2004 by Sebastien Godard <sebastien.godard@wanadoo.fr>
  */
 
 #ifndef _SA_H
@@ -13,26 +13,24 @@
 #define A_CTXSW		0x000002
 #define A_CPU		0x000004
 #define A_IRQ		0x000008
-#define A_PAGE		0x000010
+#define A_ONE_IRQ	0x000010
 #define A_SWAP		0x000020
 #define A_IO		0x000040
-/* unused */
-#define A_ONE_IRQ	0x000100
-#define A_MEMORY	0x000200
-#define A_PID		0x000400
-#define A_CPID		0x000800
-#define A_SUM_PID	0x001000
-#define A_SERIAL	0x002000
-#define A_MEM_AMT	0x004000
-/* unused */
-#define A_KTABLES	0x010000
-#define A_NET_DEV	0x020000
-#define A_NET_EDEV	0x040000
+#define A_MEMORY	0x000080
+#define A_SERIAL	0x000100
+#define A_NET_DEV	0x000200
+#define A_NET_EDEV	0x000400
+#define A_DISK		0x000800
+#define A_PID		0x001000
+#define A_CPID		0x002000
+/* 4000-8000: unused */
+#define A_PAGE		0x010000
+#define A_MEM_AMT	0x020000
+#define A_KTABLES	0x040000
 #define A_NET_SOCK	0x080000
 #define A_QUEUE		0x100000
-#define A_DISK		0x200000
 
-#define A_LAST		0x200000
+#define A_LAST		0x100000
 
 #define GET_PROC(m)	(((m) & A_PROC) == A_PROC)
 #define GET_CTXSW(m)	(((m) & A_CTXSW) == A_CTXSW)
@@ -45,7 +43,6 @@
 #define GET_MEMORY(m)	(((m) & A_MEMORY) == A_MEMORY)
 #define GET_PID(m)	(((m) & A_PID) == A_PID)
 #define GET_CPID(m)	(((m) & A_CPID) == A_CPID)
-#define GET_SUM_PID(m)	(((m) & A_SUM_PID) == A_SUM_PID)
 #define GET_CPID(m)	(((m) & A_CPID) == A_CPID)
 #define GET_ALL_PID(m)	(((m) & A_ALL_PID) == A_ALL_PID)
 #define GET_SERIAL(m)	(((m) & A_SERIAL) == A_SERIAL)
@@ -75,23 +72,27 @@
 #define K_FULL	"FULL"
 
 /* Define flags */
-#define F_ALL_PROC      0x001
-#define F_SA_ROTAT      0x002
-#define F_FLT_INC	0x004
-#define F_A_OPTION	0x008
-/* 0x10 flag used. Cf. common.h */
-#define F_H_OPTION	0x020
-#define F_ORG_TIME	0x040
-#define F_DEFAULT_COUNT	0x080
-#define F_I_OPTION	0x100
-#define F_DB_OPTION	0x200
-#define F_DO_SA_ROTAT	0x400
-#define F_PER_PROC	0x800
+#define F_ALL_PROC      0x0001
+#define F_SA_ROTAT      0x0002
+#define F_FLT_INC	0x0004
+#define F_A_OPTION	0x0008
+#define F_F_OPTION	0x0010
+#define F_H_OPTION	0x0020
+#define F_ORG_TIME	0x0040
+#define F_DEFAULT_COUNT	0x0080
+#define F_I_OPTION	0x0100
+#define F_DB_OPTION	0x0200
+#define F_DO_SA_ROTAT	0x0400
+#define F_PER_PROC	0x0800
+#define F_L_OPTION	0x1000
+#define F_FILE_LCK	0X2000
+/* 0x100000:0x800000 -> reserved (cf. common.h) */
 
 #define WANT_ALL_PROC(m)	(((m) & F_ALL_PROC) == F_ALL_PROC)
 #define WANT_SA_ROTAT(m)	(((m) & F_SA_ROTAT) == F_SA_ROTAT)
 #define FLT_ARE_INC(m)		(((m) & F_FLT_INC) == F_FLT_INC)
 #define USE_A_OPTION(m)		(((m) & F_A_OPTION) == F_A_OPTION)
+#define USE_F_OPTION(m)		(((m) & F_F_OPTION) == F_F_OPTION)
 #define USE_H_OPTION(m)		(((m) & F_H_OPTION) == F_H_OPTION)
 #define PRINT_ORG_TIME(m)	(((m) & F_ORG_TIME) == F_ORG_TIME)
 #define USE_DEFAULT_COUNT(m)	(((m) & F_DEFAULT_COUNT) == F_DEFAULT_COUNT)
@@ -99,6 +100,8 @@
 #define USE_DB_OPTION(m)	(((m) & F_DB_OPTION) == F_DB_OPTION)
 #define DO_SA_ROTAT(m)		(((m) & F_DO_SA_ROTAT) == F_DO_SA_ROTAT)
 #define WANT_PER_PROC(m)	(((m) & F_PER_PROC) == F_PER_PROC)
+#define USE_L_OPTION(m)		(((m) & F_L_OPTION) == F_L_OPTION)
+#define FILE_LOCKED(m)		(((m) & F_FILE_LCK) == F_FILE_LCK)
 
 /* Files */
 #define PROC		"/proc"
@@ -121,18 +124,18 @@
 #define SADC_PATH	"/usr/lib/sa/sadc"
 #define SADC_LOCAL_PATH	"/usr/local/lib/sa/sadc"
 #define LOADAVG		"/proc/loadavg"
+#define VMSTAT		"/proc/vmstat"
 
-#define NR_CPUS		64
-#define NR_IRQS		224
-#define MAX_IRQS_DIV_8	32	/* Assume NR_IRQS <= 256 */
+#define NR_IRQS		256
 
 #define NR_IFACE_PREALLOC	2
 #define NR_SERIAL_PREALLOC	2
 #define NR_IRQPROC_PREALLOC	3
-#define NR_DISK_PREALLOC	3
 
 /* Maximum number of processes that can be monitored simultaneously */
-#define MAX_PID_NR		256
+#define MAX_PID_NR	256
+/* Maximum length of network interface name */
+#define MAX_IFACE_LEN	16
 /*
  * Maximum number of args that can be passed to sadc:
  * sadc -x <pid> [-x <pid> ...] -X <pid> [-X <pid> ...]
@@ -147,6 +150,8 @@
 #define NO_TM_START	0
 #define NO_TM_END	0
 #define NO_RESET	0
+#define NON_FATAL	0
+#define FATAL		1
 
 /* Record type */
 #define R_STATS		1
@@ -159,13 +164,26 @@
  * System activity daily file magic number
  * (will vary when file format changes)
  */
-#define SA_MAGIC	0x215f
+#define SA_MAGIC	0x2163
 
 /*
+ * IMPORTANT NOTE:
  * Attributes such as 'aligned' and 'packed' have been defined for every
  * members of the following structures, so that:
  * 1) structures have a fixed size whether on 32 or 64-bit systems,
  * 2) we don't have variable gap between members.
+ * Indeed, we want to be able to read daily data files recorded on
+ * 32 and 64-bit systems, even if we are not on the same architecture.
+ * Moreover, we are sure that sizeof(struct) is a constant for every
+ * struct of same type, so that expressions like
+ * struct *foo2 = struct *foo1 + i;
+ * can be safely used.
+ *
+ * Structures are padded so that their length be a multiple of 8 bytes.
+ * It seems better (although not compulsory) for structures written
+ * contiguously into daily data files and accessed the following way once
+ * they are read into memory:
+ * struct *foo2 = struct *foo1 + i;  (since i <=> sizeof(struct foo))
  */
 /* System activity data file header */
 struct file_hdr {
@@ -183,6 +201,12 @@ struct file_hdr {
    unsigned long sa_ust_time			__attribute__ ((aligned (8)));
    /* Number of disks */
    unsigned int  sa_nr_disk			__attribute__ ((aligned (8)));
+   /* Number of processors: 1 means two proc */
+   unsigned int sa_proc 			__attribute__ ((packed));
+   /* Number of serial lines: 2 means two lines (ttyS00 and ttyS01) */
+   unsigned int sa_serial 			__attribute__ ((packed));
+   /* Number of network devices (interfaces): 2 means two lines */
+   unsigned int sa_iface 			__attribute__ ((packed));
    /*
     * Current day, month and year.
     * No need to save DST (daylight saving time) flag, since it is not taken
@@ -191,12 +215,6 @@ struct file_hdr {
    unsigned char sa_day				__attribute__ ((packed));
    unsigned char sa_month			__attribute__ ((packed));
    unsigned char sa_year			__attribute__ ((packed));
-   /* Number of processors: 1 means two proc */
-   unsigned char sa_proc 			__attribute__ ((packed));
-   /* Number of serial lines: 2 means two lines (ttyS00 and ttyS01) */
-   unsigned char sa_serial 			__attribute__ ((packed));
-   /* Number of network devices (interfaces): 2 means two lines */
-   unsigned char sa_iface 			__attribute__ ((packed));
    /* Operating system name */
    char          sa_sysname[UTSNAME_LEN]	__attribute__ ((packed));
    /* Machine hostname */
@@ -205,14 +223,7 @@ struct file_hdr {
    char          sa_release[UTSNAME_LEN]	__attribute__ ((packed));
 };
 
-/*
- * Note that sizeof(file_hdr) may be greater than FILE_HDR_SIZE,
- * because we use aligned(8) attributes.
- */
-#define FILE_HDR_SIZE	(SIZEOF_LONG + \
-			 sizeof(int)   * 4 + \
-                         sizeof(short) * 2 + \
-                         sizeof(char)  * (6 + 3 * UTSNAME_LEN))
+#define FILE_HDR_SIZE	(sizeof(struct file_hdr))
 
 struct file_stats {
    /* Record type: R_STATS or R_DUMMY */
@@ -225,8 +236,11 @@ struct file_stats {
    unsigned int  nr_processes			__attribute__ ((packed));
    /* Time stamp (number of seconds since the epoch) */
    unsigned long ust_time			__attribute__ ((aligned (8)));
-   /* Stats... */
+   /* Machine uptime (multiplied by the # of proc) */
    unsigned long uptime				__attribute__ ((aligned (8)));
+   /* Uptime reduced to one processor. Set *only* on SMP machines */
+   unsigned long uptime0			__attribute__ ((aligned (8)));
+   /* Stats... */
    unsigned long processes			__attribute__ ((aligned (8)));
    unsigned int  context_swtch			__attribute__ ((aligned (8)));
    unsigned int  cpu_user			__attribute__ ((packed));
@@ -234,30 +248,25 @@ struct file_stats {
    unsigned int  cpu_system			__attribute__ ((packed));
    unsigned long cpu_idle			__attribute__ ((aligned (8)));
    unsigned long cpu_iowait			__attribute__ ((aligned (8)));
-   unsigned int  irq_sum			__attribute__ ((aligned (8)));
-   unsigned int  pgpgin				__attribute__ ((packed));
-   unsigned int  pgpgout			__attribute__ ((packed));
-   unsigned int  pswpin				__attribute__ ((packed));
-   unsigned int  pswpout			__attribute__ ((packed));
-   unsigned int  dk_drive			__attribute__ ((packed));
+   unsigned long irq_sum			__attribute__ ((aligned (8)));
+   unsigned long pgpgin				__attribute__ ((aligned (8)));
+   unsigned long pgpgout			__attribute__ ((aligned (8)));
+   unsigned long pswpin				__attribute__ ((aligned (8)));
+   unsigned long pswpout			__attribute__ ((aligned (8)));
+   unsigned int  dk_drive			__attribute__ ((aligned (8)));
    unsigned int  dk_drive_rio			__attribute__ ((packed));
    unsigned int  dk_drive_wio			__attribute__ ((packed));
    unsigned int  dk_drive_rblk			__attribute__ ((packed));
    unsigned int  dk_drive_wblk			__attribute__ ((packed));
    /* Memory stats in Kb */
    unsigned long frmkb				__attribute__ ((aligned (8)));
-   unsigned long shmkb				__attribute__ ((aligned (8)));
    unsigned long bufkb				__attribute__ ((aligned (8)));
    unsigned long camkb				__attribute__ ((aligned (8)));
    unsigned long tlmkb				__attribute__ ((aligned (8)));
    unsigned long frskb				__attribute__ ((aligned (8)));
    unsigned long tlskb				__attribute__ ((aligned (8)));
-   /* minflt and majflt set only when using '-x SUM' */
-   unsigned long minflt				__attribute__ ((aligned (8)));
-   unsigned long majflt				__attribute__ ((aligned (8)));
-   unsigned int  dentry_stat			__attribute__ ((aligned (8)));
-   unsigned int  file_used			__attribute__ ((packed));
-   unsigned int  file_max			__attribute__ ((packed));
+   unsigned long caskb				__attribute__ ((aligned (8)));
+   unsigned int  file_used			__attribute__ ((aligned (8)));
    unsigned int  inode_used			__attribute__ ((packed));
    unsigned int  super_used			__attribute__ ((packed));
    unsigned int  super_max			__attribute__ ((packed));
@@ -270,22 +279,17 @@ struct file_stats {
    unsigned int  udp_inuse			__attribute__ ((packed));
    unsigned int  raw_inuse			__attribute__ ((packed));
    unsigned int  frag_inuse			__attribute__ ((packed));
-   unsigned int  nr_active_pages		__attribute__ ((packed));
-   unsigned int  nr_inactive_dirty_pages	__attribute__ ((packed));
-   unsigned int  nr_inactive_clean_pages	__attribute__ ((packed));
-   /* This field of type 'long' _must_ be aligned(8) even if the attribute
-    * here wasn't used. Else, FILE_STATS_SIZE below would be wrong! */
-   unsigned long inactive_target		__attribute__ ((aligned (8)));
-   unsigned int  load_avg_1			__attribute__ ((aligned (8)));
+   unsigned long pgfault			__attribute__ ((aligned (8)));
+   unsigned long pgmajfault			__attribute__ ((aligned (8)));
+   unsigned int  dentry_stat			__attribute__ ((aligned (8)));
+   unsigned int  load_avg_1			__attribute__ ((packed));
    unsigned int  load_avg_5			__attribute__ ((packed));
    unsigned int  load_avg_15			__attribute__ ((packed));
    unsigned int  nr_running			__attribute__ ((packed));
    unsigned int  nr_threads			__attribute__ ((packed));
 };
 
-#define FILE_STATS_SIZE	(sizeof(int)  * 38 + \
-			 sizeof(char) *  4 + \
-			 SIZEOF_LONG  * 15)
+#define FILE_STATS_SIZE	(sizeof(struct file_stats))
 
 struct stats_one_cpu {
    unsigned long per_cpu_idle			__attribute__ ((aligned (8)));
@@ -294,15 +298,14 @@ struct stats_one_cpu {
    unsigned int  per_cpu_nice			__attribute__ ((packed));
    unsigned int  per_cpu_system			__attribute__ ((packed));
    unsigned char pad[4]				__attribute__ ((packed));
-/* IMPORTANT NOTE:
+/*
+ * IMPORTANT NOTE:
  * Structure must be a multiple of 8 bytes, since we use an array of structures.
  * Each structure is *aligned*, and we want the structures to be packed together.
  */
 };
 
-#define STATS_ONE_CPU_SIZE	(sizeof(int)  * 3 + \
-				 sizeof(char) * 4 + \
-				 SIZEOF_LONG  * 2)
+#define STATS_ONE_CPU_SIZE	(sizeof(struct stats_one_cpu))
 
 struct pid_stats {
    /* If pid is null, the process has been killed */
@@ -323,20 +326,17 @@ struct pid_stats {
    /* See IMPORTANT NOTE above */
 };
 
-#define PID_STATS_SIZE	(sizeof(int) + \
-			 sizeof(char) * 4 + \
-			 SIZEOF_LONG  * 11)
+#define PID_STATS_SIZE	(sizeof(struct pid_stats))
 
 struct stats_serial {
    unsigned int  rx				__attribute__ ((aligned (8)));
    unsigned int  tx				__attribute__ ((packed));
-   unsigned char line				__attribute__ ((packed));
-   unsigned char pad[7]				__attribute__ ((packed));
+   unsigned int  line				__attribute__ ((packed));
+   unsigned char pad[4]				__attribute__ ((packed));
    /* See IMPORTANT NOTE above */
 };
 
-#define STATS_SERIAL_SIZE	(sizeof(int)  * 2 + \
-				 sizeof(char) * 8)
+#define STATS_SERIAL_SIZE	(sizeof(struct stats_serial))
 
 /* See linux source file linux/include/linux/netdevice.h */
 struct stats_net_dev {
@@ -356,13 +356,11 @@ struct stats_net_dev {
    unsigned long tx_fifo_errors			__attribute__ ((aligned (8)));
    unsigned long rx_frame_errors		__attribute__ ((aligned (8)));
    unsigned long tx_carrier_errors		__attribute__ ((aligned (8)));
-   unsigned char interface[7]			__attribute__ ((aligned (8)));
-   unsigned char pad				__attribute__ ((packed));
+   unsigned char interface[MAX_IFACE_LEN]	__attribute__ ((aligned (8)));
    /* See IMPORTANT NOTE above */
 };
 
-#define STATS_NET_DEV_SIZE	(SIZEOF_LONG  * 16 + \
-				 sizeof(char) * 8)
+#define STATS_NET_DEV_SIZE	(sizeof(struct stats_net_dev))
 
 struct stats_irq_cpu {
    unsigned int interrupt			__attribute__ ((aligned (8)));
@@ -370,29 +368,29 @@ struct stats_irq_cpu {
    /* See IMPORTANT NOTE above */
 };
 
-#define STATS_IRQ_CPU_SIZE	(sizeof(int) * 2)
+#define STATS_IRQ_CPU_SIZE	(sizeof(struct stats_irq_cpu))
 #define STATS_ONE_IRQ_SIZE	(sizeof(int) * NR_IRQS)
 
-
 struct disk_stats {
-   unsigned int major				__attribute__ ((aligned (8)));
-   unsigned int index				__attribute__ ((packed));
-   unsigned int dk_drive			__attribute__ ((packed));
-   unsigned int dk_drive_rwblk			__attribute__ ((packed));
+   unsigned int  major				__attribute__ ((aligned (8)));
+   unsigned int  index				__attribute__ ((packed));
+   unsigned int  nr_ios				__attribute__ ((packed));
+   unsigned int  rd_sect			__attribute__ ((packed));
+   unsigned int  wr_sect			__attribute__ ((packed));
+   unsigned char pad[4]				__attribute__ ((packed));
    /* See IMPORTANT NOTE above */
 };
 
-#define DISK_STATS_SIZE		(sizeof(int) * 4)
-
+#define DISK_STATS_SIZE		(sizeof(struct disk_stats))
 
 struct stats_sum {
    unsigned long count				__attribute__ ((aligned (8)));
    unsigned long frmkb				__attribute__ ((packed));
-   unsigned long shmkb				__attribute__ ((packed));
    unsigned long bufkb				__attribute__ ((packed));
    unsigned long camkb				__attribute__ ((packed));
    unsigned long frskb				__attribute__ ((packed));
    unsigned long tlskb				__attribute__ ((packed));
+   unsigned long caskb				__attribute__ ((packed));
    unsigned long dentry_stat			__attribute__ ((packed));
    unsigned long file_used			__attribute__ ((packed));
    unsigned long inode_used			__attribute__ ((packed));
@@ -404,10 +402,6 @@ struct stats_sum {
    unsigned long udp_inuse			__attribute__ ((packed));
    unsigned long raw_inuse			__attribute__ ((packed));
    unsigned long frag_inuse			__attribute__ ((packed));
-   unsigned long nr_active_pages		__attribute__ ((packed));
-   unsigned long nr_inactive_dirty_pages	__attribute__ ((packed));
-   unsigned long nr_inactive_clean_pages	__attribute__ ((packed));
-   unsigned long inactive_target		__attribute__ ((packed));
    unsigned long nr_running			__attribute__ ((packed));
    unsigned long nr_threads			__attribute__ ((packed));
    unsigned long load_avg_1			__attribute__ ((packed));
@@ -415,7 +409,7 @@ struct stats_sum {
    unsigned long load_avg_15			__attribute__ ((packed));
 };
 
-#define STATS_SUM_SIZE	(sizeof(long) * 27)
+#define STATS_SUM_SIZE	(sizeof(struct stats_sum))
 
 struct tstamp {
    int tm_sec;
@@ -428,7 +422,10 @@ struct tstamp {
 #define DEF_TMSTART	"08:00:00"
 #define DEF_TMEND	"18:00:00"
 
-#define CLOSE_ALL(_fd_)		close(_fd_[0]); \
-				close(_fd_[1])
+/* Using 'do ... while' makes this macro safer to use (trailing semicolon) */
+#define CLOSE_ALL(_fd_)		do { \
+				close(_fd_[0]); \
+				close(_fd_[1]); \
+				} while (0)
 
 #endif  /* _SA_H */
