@@ -1,6 +1,6 @@
 #!/bin/sh
 # PREFIX/lib/sa/sa1.sh
-# (C) 1999-2003 Sebastien Godard <sebastien.godard@wanadoo.fr>
+# (C) 1999-2004 Sebastien Godard <sebastien.godard@wanadoo.fr>
 #
 umask 0022
 DATE=`date +%d`
@@ -9,8 +9,8 @@ DFILE=SA_DIR/sa${DATE}
 cd ${ENDIR}
 if [ $# = 0 ]
 then
-	exec ${ENDIR}/sadc 1 1 ${DFILE}
+	exec ${ENDIR}/sadc -F -L 1 1 ${DFILE}
 else
-	exec ${ENDIR}/sadc $* ${DFILE}
+	exec ${ENDIR}/sadc -F -L $* ${DFILE}
 fi
 
